#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************

import _winreg

def getDefaultJVMPath() :
    try :
        jreKey = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, r"SOFTWARE\JavaSoft\Java Runtime Environment")
        cv = _winreg.QueryValueEx(jreKey, "CurrentVersion")
        versionKey = _winreg.OpenKey(jreKey, cv[0])
        _winreg.CloseKey(jreKey)
       
        cv = _winreg.QueryValueEx(versionKey, "RuntimeLib")
        _winreg.CloseKey(versionKey)
        
        return cv[0]
    except WindowsError, ex :
        pass
        
    return None


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import sys

import _jpype

import _jclass
import _jarray
import _jwrapper
import _jproxy
import _jexception
import _jcollection
import _jobject
import _properties
import nio
import reflect
import _refdaemon

_usePythonThreadForDaemon = False

def setUsePythonThreadForDeamon(v):
	global _usePythonThreadForDaemon
	_usePythonThreadForDaemon = v


def isJVMStarted() :
    return _jpype.isStarted()

def startJVM(jvm, *args) :
    _jpype.startup(jvm, tuple(args), True)
    _jclass._initialize()
    _jarray._initialize()
    _jwrapper._initialize()
    _jproxy._initialize()
    _jexception._initialize()
    _jcollection._initialize()
    _jobject._initialize()
    _properties._initialize()
    nio._initialize()
    reflect._initialize()
    
    # start the reference deamon thread 
    if _usePythonThreadForDaemon :
    	_refdaemon.startPython()
    else:
    	_refdaemon.startJava()
    
def attachToJVM(jvm) :
    _jpype.attach(jvm)
    
    _jclass._initialize()
    _jarray._initialize()
    _jwrapper._initialize()
    _jproxy._initialize()
    _jexception._initialize()
    _jcollection._initialize()
    _jobject._initialize()
    _properties._initialize()
        
def shutdownJVM() :
    _refdaemon.stop()
    _jpype.shutdown()
    
def isThreadAttachedToJVM() :
    return _jpype.isThreadAttachedToJVM()
    
def attachThreadToJVM() :
    _jpype.attachThreadToJVM()
    
def detachThreadFromJVM() :    
    _jpype.detachThreadFromJVM()
    
def getDefaultJVMPath() :
    if sys.platform == "win32" :
        import _windows
        return _windows.getDefaultJVMPath()
    elif sys.platform == "darwin" :
        import _darwin
        return _darwin.getDefaultJVMPath()
    else:
        import _linux
        return _linux.getDefaultJVMPath()
    
class ConversionConfigClass(object):
    def __init__(self):
        self._convertString = 1
        
    def _getConvertString(self):
        return self._convertString
    
    def _setConvertString(self, value):
        if value :
            self._convertString = 1
        else:
            self._convertString = 0
            
        _jpype.setConvertStringObjects(self._convertString)
        
    string = property(_getConvertString, _setConvertString, None )
    
ConversionConfig = ConversionConfigClass()
        


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import os
import re
import shlex
import subprocess


_KNOWN_LOCATIONS = [
        ("/opt/sun/", re.compile(r"j2sdk(.+)/jre/lib/i386/client/libjvm.so")),
        ("/usr/java/", re.compile(r"j2sdk(.+)/jre/lib/i386/client/libjvm.so")),
        ("/usr/java/", re.compile(r"jdk(.+)/jre/lib/i386/client/libjvm.so")),
]

JRE_ARCHS = ["amd64/server/libjvm.so",
             "i386/client/libjvm.so",
             "i386/server/libjvm.so", ]


def getDefaultJVMPath():
    jvm = _getJVMFromJavaHome()
    if jvm is not None:
        return jvm

    #on linux, the JVM has to be in the LD_LIBRARY_PATH anyway, so might as well inspect it first
    jvm = _getJVMFromLibPath()
    if jvm is not None:
        return jvm

    # failing that, lets look in the "known" locations
    for i in _KNOWN_LOCATIONS:
        # TODO
        pass

    return "/usr/java/jre1.5.0_05/lib/i386/client/libjvm.so"


def _getJVMFromJavaHome():
    java_home = os.getenv("JAVA_HOME")
    if java_home is None:
        try:
            # Need to use Popen since check_output is only supported in py2.7+
            java_home = subprocess.Popen(shlex.split('readlink -f /usr/bin/java'),
                                        stdout=subprocess.PIPE
                                ).communicate()[0].strip().replace('bin/java', '')
        except:  # I know. catchall is bad...
            # Avoid NoneType + String
            java_home = ''

    rootJre = None
    if os.path.exists(java_home + "/bin/javac"):
        # this is a JDK home
        rootJre = java_home + '/jre/lib'
    elif os.path.exists(java_home + "/bin/java"):
        # this is a JRE home
        rootJre = java_home + '/lib'
    else:
        return None

    for i in JRE_ARCHS:
        if os.path.exists(rootJre + "/" + i):
            return rootJre + "/" + i
    return None


def _getJVMFromLibPath():
    if 'LD_LIBRARY_PATH' in os.environ:
        libpath = os.environ['LD_LIBRARY_PATH']
        if libpath is None:
            return None

        paths = libpath.split(os.pathsep)
        for i in paths:
            if i.find('jre') != -1:
                # this could be it!
                # TODO
                pass

    return None



from jpype import *
startJVM(getDefaultJVMPath())
jarr = JArray(JFloat, 1)(10)
try:
    jarr[1:2] = [dict()] # incompatible
except Exception as e:
    print(type(e), e)
print(jarr)
#jarr[1:2] = (())
shutdownJVM()



import jpype
jvm_path = jpype.getDefaultJVMPath()
jpype.startJVM(jpype.getDefaultJVMPath(), '-Xmx256M', '-Xms64M', '-Djava.class.path=/home/marscher/sources/jpype/test/classes')
assert jpype.isJVMStarted()
cls = jpype.JPackage('jpype').io.ExceptionWhileIO('pickled_list.bin')
cls.read()



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#	   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import unittest

import jpype
import os.path
import pkgutil
import sys

def suite() :
        loader = unittest.defaultTestLoader
        if len(sys.argv) > 1:
                names = sys.argv[1:]
                test_suite = loader.loadTestsFromNames(names)
        else:
                import jpypetest
                pkgpath = os.path.dirname(jpypetest.__file__)
                names = ["jpypetest.%s" % name for _, name,
                         _ in pkgutil.iter_modules([pkgpath])]
                test_suite = loader.loadTestsFromNames(names)
	return test_suite 

def runTest() :	
	runner = unittest.TextTestRunner()
	result = runner.run(suite())
	
        if jpype.isJVMStarted():
                jpype.shutdownJVM()	
	if not result.wasSuccessful():
		sys.exit(-1)

if __name__ == '__main__' :
	runTest()



import unittest
import jpype
import os.path as path

class PropertiesTestCase(unittest.TestCase):
    def setUp(self):
        if not jpype.isJVMStarted():
            root = path.dirname(path.abspath(path.dirname(__file__)))
            jvm_path = jpype.getDefaultJVMPath()
            print "Running testsuite using JVM", jvm_path
            classpath_arg = "-Djava.class.path=%s"
            classpath_arg %= '/home/marscher/sources/jpype/test/classes'
            jpype.startJVM(jvm_path, "-ea",
                           # "-Xcheck:jni", 
                           "-Xmx256M", "-Xms64M", classpath_arg)
        self.jpype = jpype.JPackage('jpype')
    
    def testprovokateBug(self):
        arr = jpype.JArray(jpype.JInt, 1)([1,2,3])
        p = self.jpype.properties.TestProp.factory(arr)
        
if __name__ == '__main__':
    unittest.main()



import yep
from jpype import *
import numpy as np
startJVM(getDefaultJVMPath())
n = 10**7
a = np.zeros(n, dtype=np.int64)
jarr = JArray(JInt, 1)(n)
yep.start()
jarr[0:len(jarr)] = a
yep.stop()
shutdownJVM()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import codecs
import platform
from glob import glob

from setuptools import setup
from setuptools import Extension
from setuptools.command.build_ext import build_ext

def read_utf8(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    return codecs.open(filename, encoding='utf-8').read()

def find_sources():
    cpp_files = []
    for dirpath, dirnames, filenames in os.walk('src'):
        for filename in filenames:
            if filename.endswith('.cpp') or filename.endswith('.c'):
                cpp_files.append(os.path.join(dirpath, filename))
    return cpp_files

def show_error(msg):
    print('*' * 80)
    print(msg)
    print('*' * 80)

platform_specific = {
    'include_dirs': [
        os.path.join('src', 'native', 'common', 'include'),
        os.path.join('src', 'native', 'python', 'include'),
    ],
    'sources': find_sources(),
}

java_home = os.getenv('JAVA_HOME')
if sys.platform == 'win32':
    if not java_home:
        raise SystemExit('Environment variable JAVA_HOME must be set.')
    platform_specific['libraries'] = ['Advapi32']
    platform_specific['library_dir'] = [os.path.join(java_home, 'lib'), ]
    platform_specific['define_macros'] = [('WIN32', 1)]
    platform_specific['extra_compile_args'] = ['/EHsc']
    platform_specific['include_dirs'] += [
        os.path.join(java_home, 'include'),
        os.path.join(java_home, 'include', 'win32')
    ]
elif sys.platform == 'darwin':
    def getJDKIncludes(java_home):
        possible_includedirs = [os.path.join(java_home, 'Headers'),
                                os.path.join(java_home, 'include'),
                                os.path.join(java_home, 'include/darwin'),
                                # fallback
                                '/System/Library/Frameworks/JavaVM.framework/Headers']
        # make sure jni.h is found - or equivalently this java home is a JDK
        if filter(os.path.exists, [os.path.join(d, 'jni.h') for d in possible_includedirs]) == []:
            show_error('Your current java home does not point to a Java Development Kit (JDK)!\n'
                       'We were not able to find a jni.h file.\n'
                       'Tried with JAVA_HOME=%s\n' 
                       'You can either install the Java Developer package from Apple\n'
                       'OR the OpenJDK from http://oracle.com' % java_home)
            raise RuntimeError
        # return existing include dirs
        return filter(os.path.exists, possible_includedirs)

    if not java_home: # try to estimate, should work solid for osx > 10.5
        osx = platform.mac_ver()[0][:4]
        from distutils.version import StrictVersion

        # for osx > 10.5 we have the nice util /usr/libexec/java_home available
        if StrictVersion(osx) >= StrictVersion('10.6'):
            import subprocess
            # call java_home detector 
            if 'check_output' in dir(subprocess): 
            	java_home = subprocess.check_output(['/usr/libexec/java_home']).strip()
            else:
                java_home = subprocess.Popen(['/usr/libexec/java_home'], stdout=subprocess.PIPE).communicate()[0]
    else: # osx < 10.6
        java_home = '/System/Library/Frameworks/JavaVM.framework/Home/'

    platform_specific['libraries'] = ['dl']
    # this raises warning:
    # distutils/extension.py:133: UserWarning: Unknown Extension options: 'library_dir'
    #platform_specific['library_dir'] = [os.path.join(java_home, 'Libraries')]
    platform_specific['define_macros'] = [('MACOSX', 1)]
    platform_specific['include_dirs'] += getJDKIncludes(java_home)
else:
    if not java_home:
        print "No JAVA_HOME Environment Variable set. Trying to guess it..."
        # (Almost) standard in GNU/Linux
        possible_homes = glob('/usr/lib/jvm/*')
        # Sun/Oracle Java in some cases
        possible_homes += glob('/usr/java/*')
        for home in possible_homes:
            include_path = os.path.join(home, 'include')
            if os.path.exists(include_path):
                java_home = home
                break
        else:
            show_error(
                "No Java/JDK could be found. I looked in the following "
                "directories: \n\n%s\n\nPlease check that you have it "
                "installed.\n\nIf you have and the destination is not in the "
                "above list, please find out where your java's home is, "
                "set your JAVA_HOME environment variable to that path and "
                "retry the installation.\n"
                "If this still fails please open a ticket or create a "
                "pull request with a fix on github: "
                "https://github.com/originell/jpype/"
                % '\n'.join(possible_homes))
            raise RuntimeError
    platform_specific['libraries'] = ['dl']
    #platform_specific['library_dir'] = [os.path.join(java_home, 'lib')]
    platform_specific['include_dirs'] += [
        os.path.join(java_home, 'include'),
        os.path.join(java_home, 'include', 'linux'),
    ]


# try to use NumPy and define macro 'HAVE_NUMPY' if its the case, so arrays
# returned from Java can be wrapped efficiently in a ndarray.
try:
    import numpy
    if platform_specific.has_key('define_macros'):
        platform_specific['define_macros'].append(('HAVE_NUMPY', 1))
    else:
        platform_specific['define_macros'] = [('HAVE_NUMPY', 1)]

    platform_specific['include_dirs'].append(numpy.get_include())
except ImportError:
    pass

jpypeLib = Extension(name='_jpype', **platform_specific)

# omit -Wstrict-prototypes from CFLAGS since its only valid for C code.
class my_build_ext(build_ext):
    def initialize_options(self, *args):
        from distutils.sysconfig import get_config_vars
        (opt,) = get_config_vars('OPT')
        os.environ['OPT'] = ' '.join(flag for flag in opt.split() 
                                     if flag and flag != '-Wstrict-prototypes')
        build_ext.initialize_options(self)

setup(
    name='JPype1',
    version='0.5.5.4',
    description='A Python to Java bridge.',
    long_description=(read_utf8('README.rst') + '\n\n' +
                      read_utf8('doc/CHANGELOG.rst') + '\n\n' +
                      read_utf8('AUTHORS.rst')),
    license='License :: OSI Approved :: Apache Software License',
    author='Steve Menard',
    author_email='devilwolf@users.sourceforge.net',
    maintainer='Luis Nell',
    maintainer_email='cooperate@originell.org',
    url='https://github.com/originell/jpype/',
    platforms=[
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows :: Windows 7',
        'Operating System :: Microsoft :: Windows :: Windows Vista',
        'Operating System :: POSIX :: Linux',
    ],
    classifiers=[
        'Programming Language :: Java',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    packages=[
        'jpype', 'jpype.awt', 'jpype.awt.event', 'jpypex', 'jpypex.swing'],
    package_dir={
        'jpype': 'src/python/jpype',
        'jpypex': 'src/python/jpypex',
    },
    cmdclass={'build_ext': my_build_ext},
    #zip_safe=False,
    ext_modules=[jpypeLib],
)
